@@ -6,7 +6,7 @@
 from __future__ import print_function
 from functools import reduce
 
-from collections import OrderedDict
+from collections import OrderedDict, namedtuple
 import json
 import multiprocessing
 import optparse
@@ -33,6 +33,7 @@ from testrunner.testproc.rerun import RerunProc
 from testrunner.testproc.shard import ShardProc
 from testrunner.testproc.sigproc import SignalProc
 from testrunner.testproc.timeout import TimeoutProc
+from testrunner.testproc import util
 
 
 BASE_DIR = (
@@ -46,8 +47,10 @@ DEFAULT_OUT_GN = 'out.gn'
 # Map of test name synonyms to lists of test suites. Should be ordered by
 # expected runtimes (suites with slow test cases first). These groups are
 # invoked in separate steps on the bots.
+# The mapping from names used here to GN targets (which must stay in sync)
+# is defined in infra/mb/gn_isolate_map.pyl.
 TEST_MAP = {
-  # This needs to stay in sync with test/bot_default.isolate.
+  # This needs to stay in sync with group("v8_bot_default") in test/BUILD.gn.
   "bot_default": [
     "debugger",
     "mjsunit",
@@ -59,11 +62,11 @@ TEST_MAP = {
     "wasm-js",
     "fuzzer",
     "message",
-    "preparser",
     "intl",
     "unittests",
+    "wasm-api-tests",
   ],
-  # This needs to stay in sync with test/default.isolate.
+  # This needs to stay in sync with group("v8_default") in test/BUILD.gn.
   "default": [
     "debugger",
     "mjsunit",
@@ -74,20 +77,19 @@ TEST_MAP = {
     "wasm-js",
     "fuzzer",
     "message",
-    "preparser",
     "intl",
     "unittests",
+    "wasm-api-tests",
   ],
-  # This needs to stay in sync with test/d8_default.isolate.
+  # This needs to stay in sync with group("v8_d8_default") in test/BUILD.gn.
   "d8_default": [
     "debugger",
     "mjsunit",
     "webkit",
     "message",
-    "preparser",
     "intl",
   ],
-  # This needs to stay in sync with test/optimize_for_size.isolate.
+  # This needs to stay in sync with "v8_optimize_for_size" in test/BUILD.gn.
   "optimize_for_size": [
     "debugger",
     "mjsunit",
@@ -101,69 +103,56 @@ TEST_MAP = {
   ],
 }
 
-# Double the timeout for these:
-SLOW_ARCHS = ["arm",
-              "mips",
-              "mipsel",
-              "mips64",
-              "mips64el",
-              "s390",
-              "s390x",
-              "arm64"]
+# Increase the timeout for these:
+SLOW_ARCHS = [
+  "arm",
+  "arm64",
+  "mips",
+  "mipsel",
+  "mips64",
+  "mips64el",
+  "s390",
+  "s390x",
+]
 
 
-class ModeConfig(object):
-  def __init__(self, flags, timeout_scalefactor, status_mode, execution_mode):
-    self.flags = flags
-    self.timeout_scalefactor = timeout_scalefactor
-    self.status_mode = status_mode
-    self.execution_mode = execution_mode
-
+ModeConfig = namedtuple(
+    'ModeConfig', 'label flags timeout_scalefactor status_mode')
 
 DEBUG_FLAGS = ["--nohard-abort", "--enable-slow-asserts", "--verify-heap"]
 RELEASE_FLAGS = ["--nohard-abort"]
-MODES = {
-  "debug": ModeConfig(
+
+DEBUG_MODE = ModeConfig(
+    label='debug',
     flags=DEBUG_FLAGS,
     timeout_scalefactor=4,
     status_mode="debug",
-    execution_mode="debug",
-  ),
-  "optdebug": ModeConfig(
-    flags=DEBUG_FLAGS,
-    timeout_scalefactor=4,
-    status_mode="debug",
-    execution_mode="debug",
-  ),
-  "release": ModeConfig(
+)
+
+RELEASE_MODE = ModeConfig(
+    label='release',
     flags=RELEASE_FLAGS,
     timeout_scalefactor=1,
     status_mode="release",
-    execution_mode="release",
-  ),
-  # Normal trybot release configuration. There, dchecks are always on which
-  # implies debug is set. Hence, the status file needs to assume debug-like
-  # behavior/timeouts.
-  "tryrelease": ModeConfig(
+)
+
+# Normal trybot release configuration. There, dchecks are always on which
+# implies debug is set. Hence, the status file needs to assume debug-like
+# behavior/timeouts.
+TRY_RELEASE_MODE = ModeConfig(
+    label='release+dchecks',
     flags=RELEASE_FLAGS,
-    timeout_scalefactor=1,
+    timeout_scalefactor=4,
     status_mode="debug",
-    execution_mode="release",
-  ),
-  # This mode requires v8 to be compiled with dchecks and slow dchecks.
-  "slowrelease": ModeConfig(
-    flags=RELEASE_FLAGS + ["--enable-slow-asserts"],
-    timeout_scalefactor=2,
-    status_mode="debug",
-    execution_mode="release",
-  ),
-}
+)
 
 PROGRESS_INDICATORS = {
   'verbose': progress.VerboseProgressIndicator,
+  'ci': progress.CIProgressIndicator,
   'dots': progress.DotsProgressIndicator,
   'color': progress.ColorProgressIndicator,
   'mono': progress.MonochromeProgressIndicator,
+  'stream': progress.StreamProgressIndicator,
 }
 
 class TestRunnerError(Exception):
@@ -188,12 +177,10 @@ class BuildConfig(object):
     self.is_full_debug = build_config['is_full_debug']
     self.msan = build_config['is_msan']
     self.no_i18n = not build_config['v8_enable_i18n_support']
-    self.no_snap = not build_config['v8_use_snapshot']
     self.predictable = build_config['v8_enable_verify_predictable']
     self.tsan = build_config['is_tsan']
     # TODO(machenbach): We only have ubsan not ubsan_vptr.
     self.ubsan_vptr = build_config['is_ubsan_vptr']
-    self.embedded_builtins = build_config['v8_enable_embedded_builtins']
     self.verify_csa = build_config['v8_enable_verify_csa']
     self.lite_mode = build_config['v8_enable_lite_mode']
     self.pointer_compression = build_config['v8_enable_pointer_compression']
@@ -222,16 +209,12 @@ class BuildConfig(object):
       detected_options.append('msan')
     if self.no_i18n:
       detected_options.append('no_i18n')
-    if self.no_snap:
-      detected_options.append('no_snap')
     if self.predictable:
       detected_options.append('predictable')
     if self.tsan:
       detected_options.append('tsan')
     if self.ubsan_vptr:
       detected_options.append('ubsan_vptr')
-    if self.embedded_builtins:
-      detected_options.append('embedded_builtins')
     if self.verify_csa:
       detected_options.append('verify_csa')
     if self.lite_mode:
@@ -242,12 +225,29 @@ class BuildConfig(object):
     return '\n'.join(detected_options)
 
 
+def _do_load_build_config(outdir, verbose=False):
+  build_config_path = os.path.join(outdir, "v8_build_config.json")
+  if not os.path.exists(build_config_path):
+    if verbose:
+      print("Didn't find build config: %s" % build_config_path)
+    raise TestRunnerError()
+
+  with open(build_config_path) as f:
+    try:
+      build_config_json = json.load(f)
+    except Exception:  # pragma: no cover
+      print("%s exists but contains invalid json. Is your build up-to-date?"
+            % build_config_path)
+      raise TestRunnerError()
+
+  return BuildConfig(build_config_json)
+
+
 class BaseTestRunner(object):
   def __init__(self, basedir=None):
     self.basedir = basedir or BASE_DIR
     self.outdir = None
     self.build_config = None
-    self.mode_name = None
     self.mode_options = None
     self.target_os = None
 
@@ -267,6 +267,9 @@ class BaseTestRunner(object):
         # this less cryptic by printing it ourselves.
         print(' '.join(sys.argv))
 
+        # Kill stray processes from previous tasks on swarming.
+        util.kill_processes_linux()
+
       self._load_build_config(options)
       command.setup(self.target_os, options.device)
 
@@ -281,7 +284,7 @@ class BaseTestRunner(object):
       tests = self._load_testsuite_generators(args, options)
       self._setup_env()
       print(">>> Running tests for %s.%s" % (self.build_config.arch,
-                                            self.mode_name))
+                                             self.mode_options.label))
       exit_code = self._do_execute(tests, args, options)
       if exit_code == utils.EXIT_CODE_FAILURES and options.json_test_results:
         print("Force exit code 0 after failures. Json test results file "
@@ -313,13 +316,8 @@ class BaseTestRunner(object):
                       default=False, action="store_true")
     parser.add_option("--outdir", help="Base directory with compile output",
                       default="out")
-    parser.add_option("--buildbot", help="DEPRECATED!",
-                      default=False, action="store_true")
     parser.add_option("--arch",
                       help="The architecture to run tests for")
-    parser.add_option("-m", "--mode",
-                      help="The test mode in which to run (uppercase for ninja"
-                      " and buildbot builds): %s" % MODES.keys())
     parser.add_option("--shell-dir", help="DEPRECATED! Executables from build "
                       "directory will be used")
     parser.add_option("--test-root", help="Root directory of the test suites",
@@ -348,12 +346,18 @@ class BaseTestRunner(object):
                            "color, mono)")
     parser.add_option("--json-test-results",
                       help="Path to a file for storing json results.")
+    parser.add_option('--slow-tests-cutoff', type="int", default=100,
+                      help='Collect N slowest tests')
     parser.add_option("--junitout", help="File name of the JUnit output")
     parser.add_option("--junittestsuite", default="v8tests",
                       help="The testsuite name in the JUnit output file")
     parser.add_option("--exit-after-n-failures", type="int", default=100,
                       help="Exit after the first N failures instead of "
                            "running all tests. Pass 0 to disable this feature.")
+    parser.add_option("--ci-test-completion",
+                      help="Path to a file for logging test completion in the "
+                           "context of CI progress indicator. Ignored if "
+                           "progress indicator is other than 'ci'.")
 
     # Rerun
     parser.add_option("--rerun-failures-count", default=0, type=int,
@@ -380,6 +384,8 @@ class BaseTestRunner(object):
                       help="Timeout for single test in seconds")
     parser.add_option("-v", "--verbose", default=False, action="store_true",
                       help="Verbose output")
+    parser.add_option('--regenerate-expected-files', default=False, action='store_true',
+                      help='Regenerate expected files')
 
     # TODO(machenbach): Temporary options for rolling out new test runner
     # features.
@@ -396,9 +402,8 @@ class BaseTestRunner(object):
   def _parse_args(self, parser, sys_args):
     options, args = parser.parse_args(sys_args)
 
-    if any(map(lambda v: v and ',' in v,
-                [options.arch, options.mode])):  # pragma: no cover
-      print('Multiple arch/mode are deprecated')
+    if options.arch and ',' in options.arch:  # pragma: no cover
+      print('Multiple architectures are deprecated')
       raise TestRunnerError()
 
     return options, args
@@ -406,7 +411,12 @@ class BaseTestRunner(object):
   def _load_build_config(self, options):
     for outdir in self._possible_outdirs(options):
       try:
-        self.build_config = self._do_load_build_config(outdir, options.verbose)
+        self.build_config = _do_load_build_config(outdir, options.verbose)
+
+        # In auto-detect mode the outdir is always where we found the build config.
+        # This ensures that we'll also take the build products from there.
+        self.outdir = outdir
+        break
       except TestRunnerError:
         pass
 
@@ -429,8 +439,7 @@ class BaseTestRunner(object):
   # Returns possible build paths in order:
   # gn
   # outdir
-  # outdir/arch.mode
-  # Each path is provided in two versions: <path> and <path>/mode for buildbot.
+  # outdir on bots
   def _possible_outdirs(self, options):
     def outdirs():
       if options.gn:
@@ -438,17 +447,13 @@ class BaseTestRunner(object):
         return
 
       yield options.outdir
-      if options.arch and options.mode:
-        yield os.path.join(options.outdir,
-                          '%s.%s' % (options.arch, options.mode))
+
+      if os.path.basename(options.outdir) != 'build':
+        yield os.path.join(options.outdir, 'build')
 
     for outdir in outdirs():
       yield os.path.join(self.basedir, outdir)
 
-      # buildbot option
-      if options.mode:
-        yield os.path.join(self.basedir, outdir, options.mode)
-
   def _get_gn_outdir(self):
     gn_out_dir = os.path.join(self.basedir, DEFAULT_OUT_GN)
     latest_timestamp = -1
@@ -464,51 +469,13 @@ class BaseTestRunner(object):
       print(">>> Latest GN build found: %s" % latest_config)
       return os.path.join(DEFAULT_OUT_GN, latest_config)
 
-  def _do_load_build_config(self, outdir, verbose=False):
-    build_config_path = os.path.join(outdir, "v8_build_config.json")
-    if not os.path.exists(build_config_path):
-      if verbose:
-        print("Didn't find build config: %s" % build_config_path)
-      raise TestRunnerError()
-
-    with open(build_config_path) as f:
-      try:
-        build_config_json = json.load(f)
-      except Exception:  # pragma: no cover
-        print("%s exists but contains invalid json. Is your build up-to-date?"
-              % build_config_path)
-        raise TestRunnerError()
-
-    # In auto-detect mode the outdir is always where we found the build config.
-    # This ensures that we'll also take the build products from there.
-    self.outdir = os.path.dirname(build_config_path)
-
-    return BuildConfig(build_config_json)
-
   def _process_default_options(self, options):
-    # We don't use the mode for more path-magic.
-    # Therefore transform the buildbot mode here to fix build_config value.
-    if options.mode:
-      options.mode = self._buildbot_to_v8_mode(options.mode)
-
-    build_config_mode = 'debug' if self.build_config.is_debug else 'release'
-    if options.mode:
-      if options.mode not in MODES:  # pragma: no cover
-        print('%s mode is invalid' % options.mode)
-        raise TestRunnerError()
-      if MODES[options.mode].execution_mode != build_config_mode:
-        print ('execution mode (%s) for %s is inconsistent with build config '
-               '(%s)' % (
-            MODES[options.mode].execution_mode,
-            options.mode,
-            build_config_mode))
-        raise TestRunnerError()
-
-      self.mode_name = options.mode
+    if self.build_config.is_debug:
+      self.mode_options = DEBUG_MODE
+    elif self.build_config.dcheck_always_on:
+      self.mode_options = TRY_RELEASE_MODE
     else:
-      self.mode_name = build_config_mode
-
-    self.mode_options = MODES[self.mode_name]
+      self.mode_options = RELEASE_MODE
 
     if options.arch and options.arch != self.build_config.arch:
       print('--arch value (%s) inconsistent with build config (%s).' % (
@@ -529,15 +496,6 @@ class BaseTestRunner(object):
     options.command_prefix = shlex.split(options.command_prefix)
     options.extra_flags = sum(map(shlex.split, options.extra_flags), [])
 
-  def _buildbot_to_v8_mode(self, config):
-    """Convert buildbot build configs to configs understood by the v8 runner.
-
-    V8 configs are always lower case and without the additional _x64 suffix
-    for 64 bit builds on windows with ninja.
-    """
-    mode = config[:-4] if config.endswith('_x64') else config
-    return mode.lower()
-
   def _process_options(self, options):
     pass
 
@@ -561,6 +519,9 @@ class BaseTestRunner(object):
         asan_options.append('detect_leaks=1')
       else:
         asan_options.append('detect_leaks=0')
+      if utils.GuessOS() == 'windows':
+        # https://crbug.com/967663
+        asan_options.append('detect_stack_use_after_return=0')
       os.environ['ASAN_OPTIONS'] = ":".join(asan_options)
 
     if self.build_config.cfi_vptr:
@@ -672,6 +633,7 @@ class BaseTestRunner(object):
       "arch": self.build_config.arch,
       "asan": self.build_config.asan,
       "byteorder": sys.byteorder,
+      "cfi_vptr": self.build_config.cfi_vptr,
       "dcheck_always_on": self.build_config.dcheck_always_on,
       "deopt_fuzzer": False,
       "endurance_fuzzer": False,
@@ -682,13 +644,10 @@ class BaseTestRunner(object):
       "is_clang": self.build_config.is_clang,
       "is_full_debug": self.build_config.is_full_debug,
       "mips_arch_variant": mips_arch_variant,
-      "mode": self.mode_options.status_mode
-              if not self.build_config.dcheck_always_on
-              else "debug",
+      "mode": self.mode_options.status_mode,
       "msan": self.build_config.msan,
       "no_harness": options.no_harness,
       "no_i18n": self.build_config.no_i18n,
-      "no_snap": self.build_config.no_snap,
       "novfp3": False,
       "optimize_for_size": "--optimize-for-size" in options.extra_flags,
       "predictable": self.build_config.predictable,
@@ -697,19 +656,22 @@ class BaseTestRunner(object):
       "system": self.target_os,
       "tsan": self.build_config.tsan,
       "ubsan_vptr": self.build_config.ubsan_vptr,
-      "embedded_builtins": self.build_config.embedded_builtins,
       "verify_csa": self.build_config.verify_csa,
       "lite_mode": self.build_config.lite_mode,
       "pointer_compression": self.build_config.pointer_compression,
     }
 
+  def _runner_flags(self):
+    """Extra default flags specific to the test runner implementation."""
+    return []
+
   def _create_test_config(self, options):
     timeout = options.timeout * self._timeout_scalefactor(options)
     return TestConfig(
         command_prefix=options.command_prefix,
         extra_flags=options.extra_flags,
         isolates=options.isolates,
-        mode_flags=self.mode_options.flags,
+        mode_flags=self.mode_options.flags + self._runner_flags(),
         no_harness=options.no_harness,
         noi18n=self.build_config.no_i18n,
         random_seed=options.random_seed,
@@ -717,13 +679,14 @@ class BaseTestRunner(object):
         shell_dir=self.outdir,
         timeout=timeout,
         verbose=options.verbose,
+        regenerate_expected_files=options.regenerate_expected_files,
     )
 
   def _timeout_scalefactor(self, options):
     """Increases timeout for slow build configurations."""
     factor = self.mode_options.timeout_scalefactor
     if self.build_config.arch in SLOW_ARCHS:
-      factor *= 4
+      factor *= 4.5
     if self.build_config.lite_mode:
       factor *= 2
     if self.build_config.predictable:
@@ -794,11 +757,10 @@ class BaseTestRunner(object):
       procs.append(progress.JUnitTestProgressIndicator(options.junitout,
                                                        options.junittestsuite))
     if options.json_test_results:
-      procs.append(progress.JsonTestProgressIndicator(
-        self.framework_name,
-        options.json_test_results,
-        self.build_config.arch,
-        self.mode_options.execution_mode))
+      procs.append(progress.JsonTestProgressIndicator(self.framework_name))
+
+    for proc in procs:
+      proc.configure(options)
 
     for proc in procs:
       try:
