@@ -22,6 +22,7 @@
 
 const {
   Error,
+  ErrorCaptureStackTrace,
   ObjectAssign,
   ObjectIs,
   ObjectKeys,
@@ -223,8 +224,6 @@ function parseCode(code, offset) {
       require('internal/deps/acorn-plugins/acorn-private-methods/index');
     const classFields =
       require('internal/deps/acorn-plugins/acorn-class-fields/index');
-    const numericSeparator =
-      require('internal/deps/acorn-plugins/acorn-numeric-separator/index');
     const staticClassFeatures =
       require('internal/deps/acorn-plugins/acorn-static-class-features/index');
 
@@ -233,7 +232,6 @@ function parseCode(code, offset) {
     const Parser = acorn.Parser.extend(
       privateMethods,
       classFields,
-      numericSeparator,
       staticClassFeatures
     );
     parseExpressionAt = Parser.parseExpressionAt.bind(Parser);
@@ -243,7 +241,7 @@ function parseCode(code, offset) {
   // Parse the read code until the correct expression is found.
   do {
     try {
-      node = parseExpressionAt(code, start, { ecmaVersion: 11 });
+      node = parseExpressionAt(code, start, { ecmaVersion: 'latest' });
       start = node.end + 1 || start;
       // Find the CallExpression in the tree.
       node = findNodeAround(node, offset, 'CallExpression');
@@ -274,8 +272,7 @@ function getErrMessage(message, fn) {
   // We only need the stack trace. To minimize the overhead use an object
   // instead of an error.
   const err = {};
-  // eslint-disable-next-line no-restricted-syntax
-  Error.captureStackTrace(err, fn);
+  ErrorCaptureStackTrace(err, fn);
   Error.stackTraceLimit = tmpLimit;
 
   overrideStackTrace.set(err, (_, stack) => stack);
@@ -794,7 +791,7 @@ function hasMatchingError(actual, expected) {
   if (expected.prototype !== undefined && actual instanceof expected) {
     return true;
   }
-  if (Error.isPrototypeOf(expected)) {
+  if (ObjectPrototypeIsPrototypeOf(Error, expected)) {
     return false;
   }
   return expected.call({}, actual) === true;
