@@ -37,10 +37,12 @@ assertCrypto();
 
 const {
   ERR_CRYPTO_FIPS_FORCED,
+  ERR_CRYPTO_FIPS_UNAVAILABLE
 } = require('internal/errors').codes;
 const constants = internalBinding('constants').crypto;
 const { getOptionValue } = require('internal/options');
 const pendingDeprecation = getOptionValue('--pending-deprecation');
+const { fipsMode } = internalBinding('config');
 const fipsForced = getOptionValue('--force-fips');
 const {
   getFipsCrypto,
@@ -51,8 +53,7 @@ const {
   randomBytes,
   randomFill,
   randomFillSync,
-  randomInt,
-  randomUUID,
+  randomInt
 } = require('internal/crypto/random');
 const {
   pbkdf2,
@@ -62,9 +63,15 @@ const {
   scrypt,
   scryptSync
 } = require('internal/crypto/scrypt');
+const {
+  hkdf,
+  hkdfSync
+} = require('internal/crypto/hkdf');
 const {
   generateKeyPair,
-  generateKeyPairSync
+  generateKeyPairSync,
+  generateKey,
+  generateKeySync,
 } = require('internal/crypto/keygen');
 const {
   createSecretKey,
@@ -86,7 +93,8 @@ const {
   privateDecrypt,
   privateEncrypt,
   publicDecrypt,
-  publicEncrypt
+  publicEncrypt,
+  getCipherInfo,
 } = require('internal/crypto/cipher');
 const {
   Sign,
@@ -105,6 +113,7 @@ const {
   getHashes,
   setDefaultEncoding,
   setEngine,
+  lazyRequire,
 } = require('internal/crypto/util');
 const Certificate = require('internal/crypto/certificate');
 
@@ -170,13 +179,18 @@ module.exports = {
   createVerify,
   diffieHellman,
   getCiphers,
+  getCipherInfo,
   getCurves,
   getDiffieHellman: createDiffieHellmanGroup,
   getHashes,
+  hkdf,
+  hkdfSync,
   pbkdf2,
   pbkdf2Sync,
   generateKeyPair,
   generateKeyPairSync,
+  generateKey,
+  generateKeySync,
   privateDecrypt,
   privateEncrypt,
   publicDecrypt,
@@ -185,14 +199,15 @@ module.exports = {
   randomFill,
   randomFillSync,
   randomInt,
-  randomUUID,
   scrypt,
   scryptSync,
   sign: signOneShot,
   setEngine,
   timingSafeEqual,
-  getFips: fipsForced ? getFipsForced : getFipsCrypto,
-  setFips: fipsForced ? setFipsForced : setFipsCrypto,
+  getFips: !fipsMode ? getFipsDisabled :
+    fipsForced ? getFipsForced : getFipsCrypto,
+  setFips: !fipsMode ? setFipsDisabled :
+    fipsForced ? setFipsForced : setFipsCrypto,
   verify: verifyOneShot,
 
   // Classes
@@ -211,11 +226,19 @@ module.exports = {
   Verify
 };
 
+function setFipsDisabled() {
+  throw new ERR_CRYPTO_FIPS_UNAVAILABLE();
+}
+
 function setFipsForced(val) {
   if (val) return;
   throw new ERR_CRYPTO_FIPS_FORCED();
 }
 
+function getFipsDisabled() {
+  return 0;
+}
+
 function getFipsForced() {
   return 1;
 }
@@ -237,8 +260,10 @@ ObjectDefineProperties(module.exports, {
   },
   // crypto.fips is deprecated. DEP0093. Use crypto.getFips()/crypto.setFips()
   fips: {
-    get: fipsForced ? getFipsForced : getFipsCrypto,
-    set: fipsForced ? setFipsForced : setFipsCrypto
+    get: !fipsMode ? getFipsDisabled :
+      fipsForced ? getFipsForced : getFipsCrypto,
+    set: !fipsMode ? setFipsDisabled :
+      fipsForced ? setFipsForced : setFipsCrypto
   },
   DEFAULT_ENCODING: {
     enumerable: false,
@@ -254,6 +279,12 @@ ObjectDefineProperties(module.exports, {
     value: constants
   },
 
+  webcrypto: {
+    configurable: false,
+    enumerable: true,
+    get() { return lazyRequire('internal/crypto/webcrypto'); }
+  },
+
   // Aliases for randomBytes are deprecated.
   // The ecosystem needs those to exist for backwards compatibility.
   prng: {
