@@ -1,10 +1,11 @@
 'use strict';
 
 const {
-  Map,
   NumberParseInt,
   ObjectDefineProperty,
+  SafeMap,
   SafeWeakMap,
+  StringPrototypeStartsWith,
 } = primordials;
 
 const {
@@ -96,7 +97,8 @@ function patchProcessObject(expandArgv1) {
   });
   process.argv[0] = process.execPath;
 
-  if (expandArgv1 && process.argv[1] && !process.argv[1].startsWith('-')) {
+  if (expandArgv1 && process.argv[1] &&
+      !StringPrototypeStartsWith(process.argv[1], '-')) {
     // Expand process.argv[1] into a full path.
     const path = require('path');
     try {
@@ -357,7 +359,7 @@ function initializePolicy() {
   if (experimentalPolicy) {
     process.emitWarning('Policies are experimental.',
                         'ExperimentalWarning');
-    const { pathToFileURL, URL } = require('url');
+    const { pathToFileURL, URL } = require('internal/url');
     // URL here as it is slightly different parsing
     // no bare specifiers for now
     let manifestURL;
@@ -374,7 +376,7 @@ function initializePolicy() {
     if (experimentalPolicyIntegrity) {
       const SRI = require('internal/policy/sri');
       const { createHash, timingSafeEqual } = require('crypto');
-      const realIntegrities = new Map();
+      const realIntegrities = new SafeMap();
       const integrityEntries = SRI.parse(experimentalPolicyIntegrity);
       let foundMatch = false;
       for (let i = 0; i < integrityEntries.length; i++) {
