@@ -71,7 +71,7 @@ function generateKeyPair(type, options, callback) {
   if (typeof callback !== 'function')
     throw new ERR_INVALID_CALLBACK(callback);
 
-  const job = check(kCryptoJobAsync, type, options);
+  const job = createJob(kCryptoJobAsync, type, options);
 
   job.ondone = (error, result) => {
     if (error) return FunctionPrototypeCall(callback, job, error);
@@ -91,7 +91,7 @@ ObjectDefineProperty(generateKeyPair, customPromisifyArgs, {
 });
 
 function generateKeyPairSync(type, options) {
-  return handleError(check(kCryptoJobSync, type, options).run());
+  return handleError(createJob(kCryptoJobSync, type, options).run());
 }
 
 function handleError(ret) {
@@ -152,7 +152,7 @@ function parseKeyEncoding(keyType, options = {}) {
   ];
 }
 
-function check(mode, type, options) {
+function createJob(mode, type, options) {
   validateString(type, 'type');
 
   const encoding = parseKeyEncoding(type, options);
