@@ -1,7 +1,9 @@
 'use strict';
 
 const {
+  ArrayFrom,
   ObjectDefineProperty,
+  ObjectSetPrototypeOf,
   Symbol,
 } = primordials;
 
@@ -16,21 +18,47 @@ const {
   kKeyEncodingPKCS1,
   kKeyEncodingPKCS8,
   kKeyEncodingSPKI,
-  kKeyEncodingSEC1
+  kKeyEncodingSEC1,
 } = internalBinding('crypto');
+
 const {
-  ERR_CRYPTO_INCOMPATIBLE_KEY_OPTIONS,
-  ERR_CRYPTO_INVALID_KEY_OBJECT_TYPE,
-  ERR_INVALID_ARG_TYPE,
-  ERR_INVALID_ARG_VALUE,
-  ERR_INVALID_OPT_VALUE,
-  ERR_OUT_OF_RANGE
-} = require('internal/errors').codes;
-const { kHandle } = require('internal/crypto/util');
+  codes: {
+    ERR_CRYPTO_INCOMPATIBLE_KEY_OPTIONS,
+    ERR_CRYPTO_INVALID_KEY_OBJECT_TYPE,
+    ERR_INVALID_ARG_TYPE,
+    ERR_INVALID_ARG_VALUE,
+    ERR_OUT_OF_RANGE,
+    ERR_OPERATION_FAILED,
+  }
+} = require('internal/errors');
 
-const { isArrayBufferView } = require('internal/util/types');
+const {
+  kHandle,
+  kKeyObject,
+  getArrayBufferOrView,
+} = require('internal/crypto/util');
 
+const {
+  isAnyArrayBuffer,
+  isArrayBufferView,
+} = require('internal/util/types');
+
+const {
+  JSTransferable,
+  kClone,
+  kDeserialize,
+} = require('internal/worker/js_transferable');
+
+const {
+  customInspectSymbol: kInspect,
+} = require('internal/util');
+
+const { inspect } = require('internal/util/inspect');
+
+const kAlgorithm = Symbol('kAlgorithm');
+const kExtractable = Symbol('kExtractable');
 const kKeyType = Symbol('kKeyType');
+const kKeyUsages = Symbol('kKeyUsages');
 
 // Key input contexts.
 const kConsumePublic = 0;
@@ -48,12 +76,12 @@ for (const m of [[kKeyEncodingPKCS1, 'pkcs1'], [kKeyEncodingPKCS8, 'pkcs8'],
 // which requires the KeyObject base class to be implemented in C++.
 // The creation requires a callback to make sure that the NativeKeyObject
 // base class cannot exist without the other KeyObject implementations.
-const {
-  0: KeyObject,
-  1: SecretKeyObject,
-  2: PublicKeyObject,
-  3: PrivateKeyObject,
-} = createNativeKeyObjectClass((NativeKeyObject) => {
+const [
+  KeyObject,
+  SecretKeyObject,
+  PublicKeyObject,
+  PrivateKeyObject
+] = createNativeKeyObjectClass((NativeKeyObject) => {
   // Publicly visible KeyObject class.
   class KeyObject extends NativeKeyObject {
     constructor(type, handle) {
@@ -77,6 +105,12 @@ const {
     get type() {
       return this[kKeyType];
     }
+
+    static from(key) {
+      if (!isCryptoKey(key))
+        throw new ERR_INVALID_ARG_TYPE('key', 'CryptoKey', key);
+      return key[kKeyObject];
+    }
   }
 
   class SecretKeyObject extends KeyObject {
@@ -142,7 +176,7 @@ function parseKeyFormat(formatStr, defaultFormat, optionName) {
     return kKeyFormatPEM;
   else if (formatStr === 'der')
     return kKeyFormatDER;
-  throw new ERR_INVALID_OPT_VALUE(optionName, formatStr);
+  throw new ERR_INVALID_ARG_VALUE(optionName, formatStr);
 }
 
 function parseKeyType(typeStr, required, keyType, isPublic, optionName) {
@@ -166,11 +200,12 @@ function parseKeyType(typeStr, required, keyType, isPublic, optionName) {
     return kKeyEncodingSEC1;
   }
 
-  throw new ERR_INVALID_OPT_VALUE(optionName, typeStr);
+  throw new ERR_INVALID_ARG_VALUE(optionName, typeStr);
 }
 
 function option(name, objName) {
-  return objName === undefined ? name : `${objName}.${name}`;
+  return objName === undefined ?
+    `options.${name}` : `options.${objName}.${name}`;
 }
 
 function parseKeyFormatAndType(enc, keyType, isPublic, objName) {
@@ -191,7 +226,9 @@ function parseKeyFormatAndType(enc, keyType, isPublic, objName) {
 }
 
 function isStringOrBuffer(val) {
-  return typeof val === 'string' || isArrayBufferView(val);
+  return typeof val === 'string' ||
+         isArrayBufferView(val) ||
+         isAnyArrayBuffer(val);
 }
 
 function parseKeyEncoding(enc, keyType, isPublic, objName) {
@@ -205,14 +242,14 @@ function parseKeyEncoding(enc, keyType, isPublic, objName) {
     type
   } = parseKeyFormatAndType(enc, keyType, isPublic, objName);
 
-  let cipher, passphrase;
+  let cipher, passphrase, encoding;
   if (isPublic !== true) {
-    ({ cipher, passphrase } = enc);
+    ({ cipher, passphrase, encoding } = enc);
 
     if (!isInput) {
       if (cipher != null) {
         if (typeof cipher !== 'string')
-          throw new ERR_INVALID_OPT_VALUE(option('cipher', objName), cipher);
+          throw new ERR_INVALID_ARG_VALUE(option('cipher', objName), cipher);
         if (format === kKeyFormatDER &&
             (type === kKeyEncodingPKCS1 ||
              type === kKeyEncodingSEC1)) {
@@ -220,18 +257,21 @@ function parseKeyEncoding(enc, keyType, isPublic, objName) {
             encodingNames[type], 'does not support encryption');
         }
       } else if (passphrase !== undefined) {
-        throw new ERR_INVALID_OPT_VALUE(option('cipher', objName), cipher);
+        throw new ERR_INVALID_ARG_VALUE(option('cipher', objName), cipher);
       }
     }
 
     if ((isInput && passphrase !== undefined &&
          !isStringOrBuffer(passphrase)) ||
         (!isInput && cipher != null && !isStringOrBuffer(passphrase))) {
-      throw new ERR_INVALID_OPT_VALUE(option('passphrase', objName),
+      throw new ERR_INVALID_ARG_VALUE(option('passphrase', objName),
                                       passphrase);
     }
   }
 
+  if (passphrase !== undefined)
+    passphrase = getArrayBufferOrView(passphrase, 'key.passphrase', encoding);
+
   return { format, type, cipher, passphrase };
 }
 
@@ -253,7 +293,7 @@ function getKeyObjectHandle(key, ctx) {
   if (ctx === kCreatePrivate) {
     throw new ERR_INVALID_ARG_TYPE(
       'key',
-      ['string', 'Buffer', 'TypedArray', 'DataView'],
+      ['string', 'ArrayBuffer', 'Buffer', 'TypedArray', 'DataView'],
       key
     );
   }
@@ -270,38 +310,52 @@ function getKeyObjectHandle(key, ctx) {
   return key[kHandle];
 }
 
+function getKeyTypes(allowKeyObject, bufferOnly = false) {
+  return [
+    'ArrayBuffer',
+    'Buffer',
+    'TypedArray',
+    'DataView',
+    ...(!bufferOnly ? ['string'] : []),
+    ...(!bufferOnly && allowKeyObject ? ['KeyObject', 'CryptoKey'] : [])];
+}
+
 function prepareAsymmetricKey(key, ctx) {
   if (isKeyObject(key)) {
     // Best case: A key object, as simple as that.
     return { data: getKeyObjectHandle(key, ctx) };
-  } else if (typeof key === 'string' || isArrayBufferView(key)) {
+  } else if (isCryptoKey(key)) {
+    return { data: getKeyObjectHandle(key[kKeyObject], ctx) };
+  } else if (isStringOrBuffer(key)) {
     // Expect PEM by default, mostly for backward compatibility.
-    return { format: kKeyFormatPEM, data: key };
+    return { format: kKeyFormatPEM, data: getArrayBufferOrView(key, 'key') };
   } else if (typeof key === 'object') {
-    const data = key.key;
+    const { key: data, encoding } = key;
     // The 'key' property can be a KeyObject as well to allow specifying
     // additional options such as padding along with the key.
     if (isKeyObject(data))
       return { data: getKeyObjectHandle(data, ctx) };
+    else if (isCryptoKey(data))
+      return { data: getKeyObjectHandle(data[kKeyObject], ctx) };
     // Either PEM or DER using PKCS#1 or SPKI.
     if (!isStringOrBuffer(data)) {
       throw new ERR_INVALID_ARG_TYPE(
         'key.key',
-        ['string', 'Buffer', 'TypedArray', 'DataView',
-         ...(ctx !== kCreatePrivate ? ['KeyObject'] : [])],
+        getKeyTypes(ctx !== kCreatePrivate),
         data);
     }
 
     const isPublic =
       (ctx === kConsumePrivate || ctx === kCreatePrivate) ? false : undefined;
-    return { data, ...parseKeyEncoding(key, undefined, isPublic) };
+    return {
+      data: getArrayBufferOrView(data, 'key', encoding),
+      ...parseKeyEncoding(key, undefined, isPublic)
+    };
   }
   throw new ERR_INVALID_ARG_TYPE(
     'key',
-    ['string', 'Buffer', 'TypedArray', 'DataView',
-     ...(ctx !== kCreatePrivate ? ['KeyObject'] : [])],
-    key
-  );
+    getKeyTypes(ctx !== kCreatePrivate),
+    key);
 }
 
 function preparePrivateKey(key) {
@@ -312,24 +366,31 @@ function preparePublicOrPrivateKey(key) {
   return prepareAsymmetricKey(key, kConsumePublic);
 }
 
-function prepareSecretKey(key, bufferOnly = false) {
-  if (!isArrayBufferView(key) && (bufferOnly || typeof key !== 'string')) {
-    if (isKeyObject(key) && !bufferOnly) {
+function prepareSecretKey(key, encoding, bufferOnly = false) {
+  if (!bufferOnly) {
+    if (isKeyObject(key)) {
       if (key.type !== 'secret')
         throw new ERR_CRYPTO_INVALID_KEY_OBJECT_TYPE(key.type, 'secret');
       return key[kHandle];
+    } else if (isCryptoKey(key)) {
+      if (key.type !== 'secret')
+        throw new ERR_CRYPTO_INVALID_KEY_OBJECT_TYPE(key.type, 'secret');
+      return key[kKeyObject][kHandle];
     }
+  }
+  if (typeof key !== 'string' &&
+      !isArrayBufferView(key) &&
+      !isAnyArrayBuffer(key)) {
     throw new ERR_INVALID_ARG_TYPE(
       'key',
-      ['Buffer', 'TypedArray', 'DataView',
-       ...(bufferOnly ? [] : ['string', 'KeyObject'])],
+      getKeyTypes(!bufferOnly, bufferOnly),
       key);
   }
-  return key;
+  return getArrayBufferOrView(key, 'key', encoding);
 }
 
-function createSecretKey(key) {
-  key = prepareSecretKey(key, true);
+function createSecretKey(key, encoding) {
+  key = prepareSecretKey(key, encoding, true);
   if (key.byteLength === 0)
     throw new ERR_OUT_OF_RANGE('key.byteLength', '> 0', key.byteLength);
   const handle = new KeyObjectHandle();
@@ -356,21 +417,123 @@ function isKeyObject(key) {
   return key instanceof KeyObject;
 }
 
+// Our implementation of CryptoKey is a simple wrapper around a KeyObject
+// that adapts it to the standard interface. This implementation also
+// extends the JSTransferable class, allowing the CryptoKey to be cloned
+// to Workers.
+// TODO(@jasnell): Embedder environments like electron may have issues
+// here similar to other things like URL. A chromium provided CryptoKey
+// will not be recognized as a Node.js CryptoKey, and vice versa. It
+// would be fantastic if we could find a way of making those interop.
+class CryptoKey extends JSTransferable {
+  constructor() {
+    throw new ERR_OPERATION_FAILED('Illegal constructor');
+  }
+
+  [kInspect](depth, options) {
+    if (depth < 0)
+      return this;
+
+    const opts = {
+      ...options,
+      depth: options.depth == null ? null : options.depth - 1
+    };
+
+    return `CryptoKey ${inspect({
+      type: this.type,
+      extractable: this.extractable,
+      algorithm: this.algorithm,
+      usages: this.usages
+    }, opts)}`;
+  }
+
+  get type() {
+    return this[kKeyObject].type;
+  }
+
+  get extractable() {
+    return this[kExtractable];
+  }
+
+  get algorithm() {
+    return this[kAlgorithm];
+  }
+
+  get usages() {
+    return ArrayFrom(this[kKeyUsages]);
+  }
+
+  [kClone]() {
+    const keyObject = this[kKeyObject];
+    const algorithm = this.algorithm;
+    const extractable = this.extractable;
+    const usages = this.usages;
+
+    return {
+      data: {
+        keyObject,
+        algorithm,
+        usages,
+        extractable,
+      },
+      deserializeInfo: 'internal/crypto/keys:InternalCryptoKey'
+    };
+  }
+
+  [kDeserialize]({ keyObject, algorithm, usages, extractable }) {
+    this[kKeyObject] = keyObject;
+    this[kAlgorithm] = algorithm;
+    this[kKeyUsages] = usages;
+    this[kExtractable] = extractable;
+  }
+}
+
+// All internal code must use new InternalCryptoKey to create
+// CryptoKey instances. The CryptoKey class is exposed to end
+// user code but is not permitted to be constructed directly.
+class InternalCryptoKey extends JSTransferable {
+  constructor(
+    keyObject,
+    algorithm,
+    keyUsages,
+    extractable) {
+    super();
+    // Using symbol properties here currently instead of private
+    // properties because (for now) the performance penalty of
+    // private fields is still too high.
+    this[kKeyObject] = keyObject;
+    this[kAlgorithm] = algorithm;
+    this[kExtractable] = extractable;
+    this[kKeyUsages] = keyUsages;
+  }
+}
+
+InternalCryptoKey.prototype.constructor = CryptoKey;
+ObjectSetPrototypeOf(InternalCryptoKey.prototype, CryptoKey.prototype);
+
+function isCryptoKey(obj) {
+  return obj != null && obj[kKeyObject] !== undefined;
+}
+
 module.exports = {
   // Public API.
   createSecretKey,
   createPublicKey,
   createPrivateKey,
   KeyObject,
+  CryptoKey,
+  InternalCryptoKey,
 
   // These are designed for internal use only and should not be exposed.
   parsePublicKeyEncoding,
   parsePrivateKeyEncoding,
+  parseKeyEncoding,
   preparePrivateKey,
   preparePublicOrPrivateKey,
   prepareSecretKey,
   SecretKeyObject,
   PublicKeyObject,
   PrivateKeyObject,
-  isKeyObject
+  isKeyObject,
+  isCryptoKey,
 };
