@@ -2,6 +2,7 @@
 
 const {
   ObjectSetPrototypeOf,
+  ReflectApply,
 } = primordials;
 
 const {
@@ -50,7 +51,7 @@ function Sign(algorithm, options) {
   this[kHandle] = new _Sign();
   this[kHandle].init(algorithm);
 
-  Writable.call(this, options);
+  ReflectApply(Writable, this, [options]);
 }
 
 ObjectSetPrototypeOf(Sign.prototype, Writable.prototype);
@@ -164,7 +165,7 @@ function Verify(algorithm, options) {
   this[kHandle] = new _Verify();
   this[kHandle].init(algorithm);
 
-  Writable.call(this, options);
+  ReflectApply(Writable, this, [options]);
 }
 
 ObjectSetPrototypeOf(Verify.prototype, Writable.prototype);
