@@ -346,8 +346,19 @@ async function open(path, flags, mode) {
                                  flagsNumber, mode, kUsePromises));
 }
 
-async function read(handle, buffer, offset, length, position) {
-  validateBuffer(buffer);
+async function read(handle, bufferOrOptions, offset, length, position) {
+  let buffer = bufferOrOptions;
+  if (!isArrayBufferView(buffer)) {
+    if (bufferOrOptions.buffer) {
+      buffer = bufferOrOptions.buffer;
+      validateBuffer(buffer);
+    } else {
+      buffer = Buffer.alloc(16384);
+    }
+    offset = bufferOrOptions.offset || 0;
+    length = buffer.length;
+    position = bufferOrOptions.position || null;
+  }
 
   if (offset == null) {
     offset = 0;
