@@ -8,11 +8,6 @@ const {
 
 prepareMainThreadExecution();
 
-if (process.argv[1] === 'debug') {
-  process.emitWarning(
-    '`node debug` is deprecated. Please use `node inspect` instead.',
-    'DeprecationWarning', 'DEP0068');
-}
 
 markBootstrapComplete();
 
