@@ -106,10 +106,6 @@ primordials.SafeWeakSet = makeSafe(
   WeakSet,
   class SafeWeakSet extends WeakSet {}
 );
-primordials.SafePromise = makeSafe(
-  Promise,
-  class SafePromise extends Promise {}
-);
 
 // Create copies of the namespace objects
 [
