@@ -117,7 +117,8 @@ function unhandledRejection(promise, reason) {
   maybeUnhandledPromises.set(promise, {
     reason,
     uid: ++lastPromiseId,
-    warned: false
+    warned: false,
+    domain: process.domain
   });
   // This causes the promise to be referenced at least for one tick.
   pendingUnhandledRejections.push(promise);
@@ -192,26 +193,32 @@ function processPromiseRejections() {
     }
     promiseInfo.warned = true;
     const { reason, uid } = promiseInfo;
+    function emit(reason, promise, promiseInfo) {
+      if (promiseInfo.domain) {
+        return promiseInfo.domain.emit('error', reason);
+      }
+      return process.emit('unhandledRejection', reason, promise);
+    }
     switch (unhandledRejectionsMode) {
       case kStrictUnhandledRejections: {
         const err = reason instanceof Error ?
           reason : generateUnhandledRejectionError(reason);
         triggerUncaughtException(err, true /* fromPromise */);
-        const handled = process.emit('unhandledRejection', reason, promise);
+        const handled = emit(reason, promise, promiseInfo);
         if (!handled) emitUnhandledRejectionWarning(uid, reason);
         break;
       }
       case kIgnoreUnhandledRejections: {
-        process.emit('unhandledRejection', reason, promise);
+        emit(reason, promise, promiseInfo);
         break;
       }
       case kAlwaysWarnUnhandledRejections: {
-        process.emit('unhandledRejection', reason, promise);
+        emit(reason, promise, promiseInfo);
         emitUnhandledRejectionWarning(uid, reason);
         break;
       }
       case kThrowUnhandledRejections: {
-        const handled = process.emit('unhandledRejection', reason, promise);
+        const handled = emit(reason, promise, promiseInfo);
         if (!handled) {
           const err = reason instanceof Error ?
             reason : generateUnhandledRejectionError(reason);
@@ -220,7 +227,7 @@ function processPromiseRejections() {
         break;
       }
       case kWarnWithErrorCodeUnhandledRejections: {
-        const handled = process.emit('unhandledRejection', reason, promise);
+        const handled = emit(reason, promise, promiseInfo);
         if (!handled) {
           emitUnhandledRejectionWarning(uid, reason);
           process.exitCode = 1;
@@ -266,10 +273,9 @@ function generateUnhandledRejectionError(reason) {
 function listenForRejections() {
   setPromiseRejectCallback(promiseRejectHandler);
 }
-
 module.exports = {
   hasRejectionToWarn,
   setHasRejectionToWarn,
   listenForRejections,
-  processPromiseRejections
+  processPromiseRejections,
 };
