@@ -10,7 +10,7 @@ const {
 assertCrypto();
 
 const {
-  Array,
+  ArrayFrom,
   BigInt64Array,
   Boolean,
   Error,
@@ -22,6 +22,7 @@ const {
   PromiseResolve,
   Set,
   Symbol,
+  SymbolFor,
 } = primordials;
 
 const { Buffer } = require('buffer');
@@ -242,7 +243,7 @@ const kUsePreferredAddress = Symbol('kUsePreferredAddress');
 const kVersionNegotiation = Symbol('kVersionNegotiation');
 const kWriteGeneric = Symbol('kWriteGeneric');
 
-const kRejections = Symbol.for('nodejs.rejection');
+const kRejections = SymbolFor('nodejs.rejection');
 
 const kSocketUnbound = 0;
 const kSocketPending = 1;
@@ -1020,7 +1021,7 @@ class QuicSocket extends EventEmitter {
     const state = this[kInternalState];
     return customInspect(this, {
       endpoints: this.endpoints,
-      sessions: Array.from(state.sessions),
+      sessions: ArrayFrom(state.sessions),
       bound: this.bound,
       pending: this.pending,
       closing: this.closing,
@@ -1470,7 +1471,7 @@ class QuicSocket extends EventEmitter {
   }
 
   get endpoints() {
-    return Array.from(this[kInternalState].endpoints);
+    return ArrayFrom(this[kInternalState].endpoints);
   }
 
   get serverSecureContext() {
