@@ -129,9 +129,7 @@ function eos(stream, options, callback) {
 
   if (isRequest(stream)) {
     stream.on('complete', onfinish);
-    if (!willEmitClose) {
-      stream.on('abort', onclose);
-    }
+    stream.on('abort', onclose);
     if (stream.req) onrequest();
     else stream.on('request', onrequest);
   } else if (writable && !wState) { // legacy streams
@@ -140,7 +138,7 @@ function eos(stream, options, callback) {
   }
 
   // Not all streams will emit 'close' after 'aborted'.
-  if (!willEmitClose && typeof stream.aborted === 'boolean') {
+  if (typeof stream.aborted === 'boolean') {
     stream.on('aborted', onclose);
   }
 
@@ -149,7 +147,8 @@ function eos(stream, options, callback) {
   if (options.error !== false) stream.on('error', onerror);
   stream.on('close', onclose);
 
-  const closed = (
+  // _closed is for OutgoingMessage which is not a proper Writable.
+  const closed = (!wState && !rState && stream._closed === true) || (
     (wState && wState.closed) ||
     (rState && rState.closed) ||
     (wState && wState.errorEmitted) ||
