@@ -783,7 +783,12 @@ Readable.prototype.pipe = function(dest, pipeOpts) {
   dest.emit('pipe', src);
 
   // Start the flow if it hasn't been started already.
-  if (!state.flowing) {
+
+  if (dest.writableNeedDrain === true) {
+    if (state.flowing) {
+      src.pause();
+    }
+  } else if (!state.flowing) {
     debug('pipe resume');
     src.resume();
   }
