@@ -30,6 +30,7 @@ const {
   NumberParseInt,
   ObjectDefineProperty,
   ObjectSetPrototypeOf,
+  ReflectApply,
   Symbol,
 } = primordials;
 
@@ -434,6 +435,11 @@ function afterShutdown() {
 // of the other side sending a FIN.  The standard 'write after end'
 // is overly vague, and makes it seem like the user's code is to blame.
 function writeAfterFIN(chunk, encoding, cb) {
+  if (!this.writableEnded) {
+    return ReflectApply(
+      stream.Duplex.prototype.write, this, [chunk, encoding, cb]);
+  }
+
   if (typeof encoding === 'function') {
     cb = encoding;
     encoding = null;
@@ -947,7 +953,6 @@ Socket.prototype.connect = function(...args) {
   this._unrefTimer();
 
   this.connecting = true;
-  this.writable = true;
 
   if (pipe) {
     validateString(path, 'options.path');
