@@ -291,6 +291,11 @@ Interface.prototype.setPrompt = function(prompt) {
 };
 
 
+Interface.prototype.getPrompt = function() {
+  return this._prompt;
+};
+
+
 Interface.prototype._setRawMode = function(mode) {
   const wasInRawMode = this.input.isRaw;
 
