@@ -43,7 +43,6 @@
 'use strict';
 
 const {
-  ArrayPrototypeForEach,
   Error,
   MathMax,
   NumberIsNaN,
@@ -59,7 +58,6 @@ const {
   Promise,
   PromiseRace,
   RegExp,
-  RegExpPrototypeTest,
   Set,
   StringPrototypeCharAt,
   StringPrototypeIncludes,
@@ -68,7 +66,6 @@ const {
   SyntaxError,
   SyntaxErrorPrototype,
   WeakSet,
-  globalThis,
 } = primordials;
 
 const {
@@ -270,7 +267,6 @@ function REPLServer(prompt,
     configurable: true
   });
 
-  this.allowBlockingCompletions = !!options.allowBlockingCompletions;
   this.useColors = !!options.useColors;
   this._domain = options.domain || domain.create();
   this.useGlobal = !!useGlobal;
@@ -317,16 +313,6 @@ function REPLServer(prompt,
 
   domainSet.add(this._domain);
 
-  let rli = this;
-  ObjectDefineProperty(this, 'rli', {
-    get: deprecate(() => rli,
-                   'REPLServer.rli is deprecated', 'DEP0124'),
-    set: deprecate((val) => rli = val,
-                   'REPLServer.rli is deprecated', 'DEP0124'),
-    enumerable: true,
-    configurable: true
-  });
-
   const savedRegExMatches = ['', '', '', '', '', '', '', '', '', ''];
   const sep = '\u0000\u0000\u0000';
   const regExMatcher = new RegExp(`^${sep}(.*)${sep}(.*)${sep}(.*)${sep}(.*)` +
@@ -612,7 +598,7 @@ function REPLServer(prompt,
         errStack = self.writer(e);
 
         // Remove one line error braces to keep the old style in place.
-        if (errStack[0] === '[' && errStack[errStack.length - 1] === ']') {
+        if (errStack[errStack.length - 1] === ']') {
           errStack = errStack.slice(1, -1);
         }
       }
@@ -662,15 +648,6 @@ function REPLServer(prompt,
   });
 
   self.clearBufferedCommand();
-  ObjectDefineProperty(this, 'bufferedCommand', {
-    get: deprecate(() => self[kBufferedCommandSymbol],
-                   'REPLServer.bufferedCommand is deprecated',
-                   'DEP0074'),
-    set: deprecate((val) => self[kBufferedCommandSymbol] = val,
-                   'REPLServer.bufferedCommand is deprecated',
-                   'DEP0074'),
-    enumerable: true
-  });
 
   function completer(text, cb) {
     complete.call(self, text, self.editorMode ?
@@ -724,11 +701,6 @@ function REPLServer(prompt,
     return false;
   }
 
-  self.parseREPLKeyword = deprecate(
-    _parseREPLKeyword,
-    'REPLServer.parseREPLKeyword() is deprecated',
-    'DEP0075');
-
   self.on('close', function emitExit() {
     if (paused) {
       pausedBuffer.push(['close']);
@@ -863,11 +835,8 @@ function REPLServer(prompt,
         self.output.write(self.writer(ret) + '\n');
       }
 
-      // Display prompt again (unless we already did by emitting the 'error'
-      // event on the domain instance).
-      if (!e) {
-        self.displayPrompt();
-      }
+      // Display prompt again
+      self.displayPrompt();
     }
   });
 
@@ -986,7 +955,7 @@ REPLServer.prototype.close = function close() {
 REPLServer.prototype.createContext = function() {
   let context;
   if (this.useGlobal) {
-    context = globalThis;
+    context = global;
   } else {
     sendInspectorCommand((session) => {
       session.post('Runtime.enable');
@@ -998,13 +967,13 @@ REPLServer.prototype.createContext = function() {
     }, () => {
       context = vm.createContext();
     });
-    ArrayPrototypeForEach(ObjectGetOwnPropertyNames(globalThis), (name) => {
+    for (const name of ObjectGetOwnPropertyNames(global)) {
       // Only set properties that do not already exist as a global builtin.
       if (!globalBuiltins.has(name)) {
         ObjectDefineProperty(context, name,
-                             ObjectGetOwnPropertyDescriptor(globalThis, name));
+                             ObjectGetOwnPropertyDescriptor(global, name));
       }
-    });
+    }
     context.global = context;
     const _console = new Console(this.output);
     ObjectDefineProperty(context, 'console', {
@@ -1090,11 +1059,6 @@ REPLServer.prototype.setPrompt = function setPrompt(prompt) {
   Interface.prototype.setPrompt.call(this, prompt);
 };
 
-REPLServer.prototype.turnOffEditorMode = deprecate(
-  function() { _turnOffEditorMode(this); },
-  'REPLServer.turnOffEditorMode() is deprecated',
-  'DEP0078');
-
 const requireRE = /\brequire\s*\(\s*['"`](([\w@./-]+\/)?(?:[\w@./-]*))(?![^'"`])$/;
 const fsAutoCompleteRE = /fs(?:\.promises)?\.\s*[a-z][a-zA-Z]+\(\s*["'](.*)/;
 const simpleExpressionRE =
@@ -1193,8 +1157,7 @@ function complete(line, callback) {
     if (completeOn.length) {
       filter = completeOn;
     }
-  } else if (RegExpPrototypeTest(requireRE, line) &&
-             this.allowBlockingCompletions) {
+  } else if (requireRE.test(line)) {
     // require('...<Tab>')
     const extensions = ObjectKeys(this.context.require.extensions);
     const indexes = extensions.map((extension) => `index${extension}`);
@@ -1252,8 +1215,7 @@ function complete(line, callback) {
     if (!subdir) {
       completionGroups.push(_builtinLibs);
     }
-  } else if (RegExpPrototypeTest(fsAutoCompleteRE, line) &&
-             this.allowBlockingCompletions) {
+  } else if (fsAutoCompleteRE.test(line)) {
     [completionGroups, completeOn] = completeFSFunctions(line);
   // Handle variable member lookup.
   // We support simple chained expressions like the following (no function
@@ -1415,11 +1377,6 @@ REPLServer.prototype.defineCommand = function(keyword, cmd) {
   this.commands[keyword] = cmd;
 };
 
-REPLServer.prototype.memory = deprecate(
-  _memory,
-  'REPLServer.memory() is deprecated',
-  'DEP0082');
-
 // TODO(BridgeAR): This should be replaced with acorn to build an AST. The
 // language became more complex and using a simple approach like this is not
 // sufficient anymore.
