@@ -21,10 +21,21 @@
 
 'use strict';
 
+const {
+  ObjectDefineProperty,
+} = primordials;
+
+const {
+  promisify: { custom: customPromisify },
+} = require('internal/util');
+
 const pipeline = require('internal/streams/pipeline');
 const eos = require('internal/streams/end-of-stream');
 const internalBuffer = require('internal/buffer');
 
+// Lazy loaded
+let promises = null;
+
 const Stream = module.exports = require('internal/streams/legacy').Stream;
 Stream.Readable = require('internal/streams/readable');
 Stream.Writable = require('internal/streams/writable');
@@ -34,6 +45,33 @@ Stream.PassThrough = require('internal/streams/passthrough');
 Stream.pipeline = pipeline;
 Stream.finished = eos;
 
+function lazyLoadPromises() {
+  if (promises === null) promises = require('stream/promises');
+  return promises;
+}
+
+ObjectDefineProperty(Stream, 'promises', {
+  configurable: true,
+  enumerable: true,
+  get() {
+    return lazyLoadPromises();
+  }
+});
+
+ObjectDefineProperty(pipeline, customPromisify, {
+  enumerable: true,
+  get() {
+    return lazyLoadPromises().pipeline;
+  }
+});
+
+ObjectDefineProperty(eos, customPromisify, {
+  enumerable: true,
+  get() {
+    return lazyLoadPromises().finished;
+  }
+});
+
 // Backwards-compat with node 0.4.x
 Stream.Stream = Stream;
 
