@@ -1,6 +1,7 @@
 'use strict';
 
 const {
+  FunctionPrototypeBind,
   Promise,
   PromisePrototypeFinally,
   PromiseReject,
@@ -72,8 +73,9 @@ function setTimeout(after, value, options = {}) {
     if (!ref) timeout.unref();
     insert(timeout, timeout._idleTimeout);
     if (signal) {
-      // eslint-disable-next-line no-undef
-      oncancel = cancelListenerHandler.bind(timeout, clearTimeout, reject);
+      oncancel = FunctionPrototypeBind(cancelListenerHandler,
+                                       // eslint-disable-next-line no-undef
+                                       timeout, clearTimeout, reject);
       signal.addEventListener('abort', oncancel);
     }
   });
@@ -121,8 +123,9 @@ function setImmediate(value, options = {}) {
     const immediate = new Immediate(resolve, [value]);
     if (!ref) immediate.unref();
     if (signal) {
-      // eslint-disable-next-line no-undef
-      oncancel = cancelListenerHandler.bind(immediate, clearImmediate, reject);
+      oncancel = FunctionPrototypeBind(cancelListenerHandler,
+                                       // eslint-disable-next-line no-undef
+                                       immediate, clearImmediate, reject);
       signal.addEventListener('abort', oncancel);
     }
   });
