@@ -1,6 +1,7 @@
 (function() {
     "use strict";
     var idCounter = 0;
+    let testharness_context = null;
 
     function getInViewCenterPoint(rect) {
         var left = Math.max(0, rect.left);
@@ -15,7 +16,8 @@
     }
 
     function getPointerInteractablePaintTree(element) {
-        if (!window.document.contains(element)) {
+        let elementDocument = element.ownerDocument;
+        if (!elementDocument.contains(element)) {
             return [];
         }
 
@@ -27,10 +29,10 @@
 
         var centerPoint = getInViewCenterPoint(rectangles[0]);
 
-        if ("elementsFromPoint" in document) {
-            return document.elementsFromPoint(centerPoint[0], centerPoint[1]);
-        } else if ("msElementsFromPoint" in document) {
-            var rv = document.msElementsFromPoint(centerPoint[0], centerPoint[1]);
+        if ("elementsFromPoint" in elementDocument) {
+            return elementDocument.elementsFromPoint(centerPoint[0], centerPoint[1]);
+        } else if ("msElementsFromPoint" in elementDocument) {
+            var rv = elementDocument.msElementsFromPoint(centerPoint[0], centerPoint[1]);
             return Array.prototype.slice.call(rv ? rv : []);
         } else {
             throw new Error("document.elementsFromPoint unsupported");
@@ -47,6 +49,31 @@
      * @namespace
      */
     window.test_driver = {
+        /**
+         * Set the context in which testharness.js is loaded
+         *
+         * @param {WindowProxy} context - the window containing testharness.js
+         **/
+        set_test_context: function(context) {
+          if (window.test_driver_internal.set_test_context) {
+            window.test_driver_internal.set_test_context(context);
+          }
+          testharness_context = context;
+        },
+
+        /**
+         * postMessage to the context containing testharness.js
+         *
+         * @param {Object} msg - the data to POST
+         **/
+        message_test: function(msg) {
+            let target = testharness_context;
+            if (testharness_context === null) {
+                target = window;
+            }
+            target.postMessage(msg, "*");
+        },
+
         /**
          * Trigger user interaction in order to grant additional privileges to
          * a provided function.
@@ -56,30 +83,35 @@
          * @param {String} intent - a description of the action which much be
          *                          triggered by user interaction
          * @param {Function} action - code requiring escalated privileges
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
          *
          * @returns {Promise} fulfilled following user interaction and
          *                    execution of the provided `action` function;
          *                    rejected if interaction fails or the provided
          *                    function throws an error
          */
-        bless: function(intent, action) {
-            var button = document.createElement("button");
+        bless: function(intent, action, context=null) {
+            let contextDocument = context ? context.document : document;
+            var button = contextDocument.createElement("button");
             button.innerHTML = "This test requires user interaction.<br />" +
                 "Please click here to allow " + intent + ".";
             button.id = "wpt-test-driver-bless-" + (idCounter += 1);
-            const elem = document.body || document.documentElement;
+            const elem = contextDocument.body || contextDocument.documentElement;
             elem.appendChild(button);
 
-            return new Promise(function(resolve, reject) {
-                    button.addEventListener("click", resolve);
+            let wait_click = new Promise(resolve => button.addEventListener("click", resolve));
 
-                    test_driver.click(button).catch(reject);
-                }).then(function() {
+            return test_driver.click(button)
+                .then(wait_click)
+                .then(function() {
                     button.remove();
 
                     if (typeof action === "function") {
                         return action();
                     }
+                    return null;
                 });
         },
 
@@ -95,14 +127,6 @@
          *                    the cases the WebDriver command errors
          */
         click: function(element) {
-            if (window.top !== window) {
-                return Promise.reject(new Error("can only click in top-level window"));
-            }
-
-            if (!window.document.contains(element)) {
-                return Promise.reject(new Error("element in different document or shadow tree"));
-            }
-
             if (!inView(element)) {
                 element.scrollIntoView({behavior: "instant",
                                         block: "end",
@@ -135,14 +159,6 @@
          *                    the cases the WebDriver command errors
          */
         send_keys: function(element, keys) {
-            if (window.top !== window) {
-                return Promise.reject(new Error("can only send keys in top-level window"));
-            }
-
-            if (!window.document.contains(element)) {
-                return Promise.reject(new Error("element in different document or shadow tree"));
-            }
-
             if (!inView(element)) {
                 element.scrollIntoView({behavior: "instant",
                                         block: "end",
@@ -166,34 +182,42 @@
          * https://github.com/WICG/page-lifecycle/blob/master/README.md|Lifecycle API
          * for Web Pages}
          *
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
+         *
          * @returns {Promise} fulfilled after the freeze request is sent, or rejected
          *                    in case the WebDriver command errors
          */
-        freeze: function() {
+        freeze: function(context=null) {
             return window.test_driver_internal.freeze();
         },
 
         /**
          * Send a sequence of actions
          *
-         * This function sends a sequence of actions to the top level window
+         * This function sends a sequence of actions
          * to perform. It is modeled after the behaviour of {@link
          * https://w3c.github.io/webdriver/#actions|WebDriver Actions Command}
          *
          * @param {Array} actions - an array of actions. The format is the same as the actions
-                                    property of the WebDriver command {@link
-                                    https://w3c.github.io/webdriver/#perform-actions|Perform
-                                    Actions} command. Each element is an object representing an
-                                    input source and each input source itself has an actions
-                                    property detailing the behaviour of that source at each timestep
-                                    (or tick). Authors are not expected to construct the actions
-                                    sequence by hand, but to use the builder api provided in
-                                    testdriver-actions.js
+         *                          property of the WebDriver command {@link
+         *                          https://w3c.github.io/webdriver/#perform-actions|Perform
+         *                          Actions} command. Each element is an object representing an
+         *                          input source and each input source itself has an actions
+         *                          property detailing the behaviour of that source at each timestep
+         *                          (or tick). Authors are not expected to construct the actions
+         *                          sequence by hand, but to use the builder api provided in
+         *                          testdriver-actions.js
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
+         *
          * @returns {Promise} fufiled after the actions are performed, or rejected in
          *                    the cases the WebDriver command errors
          */
-        action_sequence: function(actions) {
-            return window.test_driver_internal.action_sequence(actions);
+        action_sequence: function(actions, context=null) {
+            return window.test_driver_internal.action_sequence(actions, context);
         },
 
         /**
@@ -203,11 +227,15 @@
          * by ReportingObserver) for testing purposes, as described in
          * {@link https://w3c.github.io/reporting/#generate-test-report-command}
          *
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
+         *
          * @returns {Promise} fulfilled after the report is generated, or
          *                    rejected if the report generation fails
          */
-        generate_test_report: function(message) {
-            return window.test_driver_internal.generate_test_report(message);
+        generate_test_report: function(message, context=null) {
+            return window.test_driver_internal.generate_test_report(message, context);
         },
 
         /**
@@ -221,6 +249,9 @@
          *                              object
          * @param {String} state - the state of the permission
          * @param {boolean} one_realm - Optional. Whether the permission applies to only one realm
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
          *
          * The above params are used to create a [PermissionSetParameters]{@link
          * https://w3c.github.io/permissions/#dictdef-permissionsetparameters} object
@@ -228,13 +259,13 @@
          * @returns {Promise} fulfilled after the permission is set, or rejected if setting the
          *                    permission fails
          */
-        set_permission: function(descriptor, state, one_realm) {
+        set_permission: function(descriptor, state, one_realm, context=null) {
             let permission_params = {
               descriptor,
               state,
               oneRealm: one_realm,
             };
-            return window.test_driver_internal.set_permission(permission_params);
+            return window.test_driver_internal.set_permission(permission_params, context);
         },
 
         /**
@@ -247,12 +278,16 @@
          * @param {Object} config - an [Authenticator Configuration]{@link
          *                          https://w3c.github.io/webauthn/#authenticator-configuration}
          *                          object
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
+         *
          * @returns {Promise} fulfilled after the authenticator is added, or
          *                    rejected in the cases the WebDriver command
          *                    errors. Returns the ID of the authenticator
          */
-        add_virtual_authenticator: function(config) {
-            return window.test_driver_internal.add_virtual_authenticator(config);
+        add_virtual_authenticator: function(config, context=null) {
+            return window.test_driver_internal.add_virtual_authenticator(config, context);
         },
 
         /**
@@ -264,13 +299,16 @@
          *
          * @param {String} authenticator_id - the ID of the authenticator to be
          *                                    removed.
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
          *
          * @returns {Promise} fulfilled after the authenticator is removed, or
          *                    rejected in the cases the WebDriver command
          *                    errors
          */
-        remove_virtual_authenticator: function(authenticator_id) {
-            return window.test_driver_internal.remove_virtual_authenticator(authenticator_id);
+        remove_virtual_authenticator: function(authenticator_id, context=null) {
+            return window.test_driver_internal.remove_virtual_authenticator(authenticator_id, context);
         },
 
         /**
@@ -282,13 +320,16 @@
          * @param {Object} credential - A [Credential Parameters]{@link
          *                              https://w3c.github.io/webauthn/#credential-parameters}
          *                              object
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
          *
          * @returns {Promise} fulfilled after the credential is added, or
          *                    rejected in the cases the WebDriver command
          *                    errors
          */
-        add_credential: function(authenticator_id, credential) {
-            return window.test_driver_internal.add_credential(authenticator_id, credential);
+        add_credential: function(authenticator_id, credential, context=null) {
+            return window.test_driver_internal.add_credential(authenticator_id, credential, context);
         },
 
         /**
@@ -300,6 +341,9 @@
          * https://w3c.github.io/webauthn/#sctn-automation-get-credentials
          *
          * @param {String} authenticator_id - the ID of the authenticator
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
          *
          * @returns {Promise} fulfilled after the credentials are returned, or
          *                    rejected in the cases the WebDriver command
@@ -307,8 +351,8 @@
          *                    Parameters]{@link
          *                    https://w3c.github.io/webauthn/#credential-parameters}
          */
-        get_credentials: function(authenticator_id) {
-            return window.test_driver_internal.get_credentials(authenticator_id);
+        get_credentials: function(authenticator_id, context=null) {
+            return window.test_driver_internal.get_credentials(authenticator_id, context=null);
         },
 
         /**
@@ -318,13 +362,16 @@
          *
          * @param {String} authenticator_id - the ID of the authenticator
          * @param {String} credential_id - the ID of the credential
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
          *
          * @returns {Promise} fulfilled after the credential is removed, or
          *                    rejected in the cases the WebDriver command
          *                    errors.
          */
-        remove_credential: function(authenticator_id, credential_id) {
-            return window.test_driver_internal.remove_credential(authenticator_id, credential_id);
+        remove_credential: function(authenticator_id, credential_id, context=null) {
+            return window.test_driver_internal.remove_credential(authenticator_id, credential_id, context);
         },
 
         /**
@@ -333,13 +380,16 @@
          * https://w3c.github.io/webauthn/#sctn-automation-remove-all-credentials
          *
          * @param {String} authenticator_id - the ID of the authenticator
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
          *
          * @returns {Promise} fulfilled after the credentials are removed, or
          *                    rejected in the cases the WebDriver command
          *                    errors.
          */
-        remove_all_credentials: function(authenticator_id) {
-            return window.test_driver_internal.remove_all_credentials(authenticator_id);
+        remove_all_credentials: function(authenticator_id, context=null) {
+            return window.test_driver_internal.remove_all_credentials(authenticator_id, context);
         },
 
         /**
@@ -351,9 +401,12 @@
          *
          * @param {String} authenticator_id - the ID of the authenticator
          * @param {boolean} uv - the User Verified flag
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
          */
-        set_user_verified: function(authenticator_id, uv) {
-            return window.test_driver_internal.set_user_verified(authenticator_id, uv);
+        set_user_verified: function(authenticator_id, uv, context=null) {
+            return window.test_driver_internal.set_user_verified(authenticator_id, uv, context);
         },
 
         /**
@@ -370,16 +423,19 @@
          *                                    May be "*" to indicate all origins.
          * @param {String} state - The storage access setting.
          *                         Must be either "allowed" or "blocked".
+         * @param {WindowProxy} context - Browsing context in which
+         *                                to run the call, or null for the current
+         *                                browsing context.
          *
          * @returns {Promise} Fulfilled after the storage access rule has been
          *                    set, or rejected if setting the rule fails.
          */
-        set_storage_access: function(origin, embedding_origin, state) {
+        set_storage_access: function(origin, embedding_origin, state, context=null) {
             if (state !== "allowed" && state !== "blocked") {
                 throw new Error("storage access status must be 'allowed' or 'blocked'");
             }
             const blocked = state === "blocked";
-            return window.test_driver_internal.set_storage_access(origin, embedding_origin, blocked);
+            return window.test_driver_internal.set_storage_access(origin, embedding_origin, blocked, context);
         },
     };
 
@@ -392,13 +448,6 @@
          */
         in_automation: false,
 
-        /**
-         * Waits for a user-initiated click
-         *
-         * @param {Element} element - element to be clicked
-         * @param {{x: number, y: number} coords - viewport coordinates to click at
-         * @returns {Promise} fulfilled after click occurs
-         */
         click: function(element, coords) {
             if (this.in_automation) {
                 return Promise.reject(new Error('Not implemented'));
@@ -409,14 +458,6 @@
             });
         },
 
-        /**
-         * Waits for an element to receive a series of key presses
-         *
-         * @param {Element} element - element which should receve key presses
-         * @param {String} keys - keys to expect
-         * @returns {Promise} fulfilled after keys are received or rejected if
-         *                    an incorrect key sequence is received
-         */
         send_keys: function(element, keys) {
             if (this.in_automation) {
                 return Promise.reject(new Error('Not implemented'));
@@ -449,158 +490,52 @@
             });
         },
 
-        /**
-         * Freeze the current page
-         *
-         * @returns {Promise} fulfilled after freeze request is sent, otherwise
-         * it gets rejected
-         */
-        freeze: function() {
+        freeze: function(context=null) {
             return Promise.reject(new Error("unimplemented"));
         },
 
-        /**
-         * Send a sequence of pointer actions
-         *
-         * @returns {Promise} fufilled after actions are sent, rejected if any actions
-         *                    fail
-         */
-        action_sequence: function(actions) {
+        action_sequence: function(actions, context=null) {
             return Promise.reject(new Error("unimplemented"));
         },
 
-        /**
-         * Generates a test report on the current page
-         *
-         * @param {String} message - the message to be contained in the report
-         * @returns {Promise} fulfilled after the report is generated, or
-         *                    rejected if the report generation fails
-         */
-        generate_test_report: function(message) {
+        generate_test_report: function(message, context=null) {
             return Promise.reject(new Error("unimplemented"));
         },
 
 
-        /**
-         * Sets the state of a permission
-         *
-         * This function simulates a user setting a permission into a particular state as described
-         * in {@link https://w3c.github.io/permissions/#set-permission-command}
-         *
-         * @param {Object} permission_params - a [PermissionSetParameters]{@lint
-         *                                     https://w3c.github.io/permissions/#dictdef-permissionsetparameters}
-         *                                     object
-         * @returns {Promise} fulfilled after the permission is set, or rejected if setting the
-         *                    permission fails
-         */
-        set_permission: function(permission_params) {
+        set_permission: function(permission_params, context=null) {
             return Promise.reject(new Error("unimplemented"));
         },
 
-        /**
-         * Creates a virtual authenticator
-         *
-         * @param {Object} config - the authenticator configuration
-         * @returns {Promise} fulfilled after the authenticator is added, or
-         *                    rejected in the cases the WebDriver command
-         *                    errors.
-         */
-        add_virtual_authenticator: function(config) {
+        add_virtual_authenticator: function(config, context=null) {
             return Promise.reject(new Error("unimplemented"));
         },
 
-        /**
-         * Removes a virtual authenticator
-         *
-         * @param {String} authenticator_id - the ID of the authenticator to be
-         *                                    removed.
-         *
-         * @returns {Promise} fulfilled after the authenticator is removed, or
-         *                    rejected in the cases the WebDriver command
-         *                    errors
-         */
-        remove_virtual_authenticator: function(authenticator_id) {
+        remove_virtual_authenticator: function(authenticator_id, context=null) {
             return Promise.reject(new Error("unimplemented"));
         },
 
-        /**
-         * Adds a credential to a virtual authenticator
-         *
-         * @param {String} authenticator_id - the ID of the authenticator
-         * @param {Object} credential - A [Credential Parameters]{@link
-         *                              https://w3c.github.io/webauthn/#credential-parameters}
-         *                              object
-         *
-         * @returns {Promise} fulfilled after the credential is added, or
-         *                    rejected in the cases the WebDriver command
-         *                    errors
-         *
-         */
-        add_credential: function(authenticator_id, credential) {
+        add_credential: function(authenticator_id, credential, context=null) {
             return Promise.reject(new Error("unimplemented"));
         },
 
-        /**
-         * Gets all the credentials stored in an authenticator
-         *
-         * @param {String} authenticator_id - the ID of the authenticator
-         *
-         * @returns {Promise} fulfilled after the credentials are returned, or
-         *                    rejected in the cases the WebDriver command
-         *                    errors. Returns an array of [Credential
-         *                    Parameters]{@link
-         *                    https://w3c.github.io/webauthn/#credential-parameters}
-         *
-         */
-        get_credentials: function(authenticator_id) {
+        get_credentials: function(authenticator_id, context=null) {
             return Promise.reject(new Error("unimplemented"));
         },
 
-        /**
-         * Remove a credential stored in an authenticator
-         *
-         * @param {String} authenticator_id - the ID of the authenticator
-         * @param {String} credential_id - the ID of the credential
-         *
-         * @returns {Promise} fulfilled after the credential is removed, or
-         *                    rejected in the cases the WebDriver command
-         *                    errors.
-         *
-         */
-        remove_credential: function(authenticator_id, credential_id) {
+        remove_credential: function(authenticator_id, credential_id, context=null) {
             return Promise.reject(new Error("unimplemented"));
         },
 
-        /**
-         * Removes all the credentials stored in a virtual authenticator
-         *
-         * @param {String} authenticator_id - the ID of the authenticator
-         *
-         * @returns {Promise} fulfilled after the credentials are removed, or
-         *                    rejected in the cases the WebDriver command
-         *                    errors.
-         *
-         */
-        remove_all_credentials: function(authenticator_id) {
+        remove_all_credentials: function(authenticator_id, context=null) {
             return Promise.reject(new Error("unimplemented"));
         },
 
-        /**
-         * Sets the User Verified flag on an authenticator
-         *
-         * @param {String} authenticator_id - the ID of the authenticator
-         * @param {boolean} uv - the User Verified flag
-         *
-         */
-        set_user_verified: function(authenticator_id, uv) {
+        set_user_verified: function(authenticator_id, uv, context=null) {
             return Promise.reject(new Error("unimplemented"));
         },
 
-        /**
-         * Sets the storage access policy for a third-party origin when loaded
-         * in the current first party context
-         */
-        set_storage_access: function(origin, embedding_origin, blocked) {
+        set_storage_access: function(origin, embedding_origin, blocked, context=null) {
             return Promise.reject(new Error("unimplemented"));
         },
     };
