@@ -49,6 +49,8 @@ const expectedModules = new Set([
   'NativeModule internal/fixed_queue',
   'NativeModule internal/fs/dir',
   'NativeModule internal/fs/utils',
+  'NativeModule internal/fs/promises',
+  'NativeModule internal/fs/rimraf',
   'NativeModule internal/idna',
   'NativeModule internal/linkedlist',
   'NativeModule internal/modules/run_main',
