@@ -7,6 +7,7 @@ if (!common.hasCrypto)
 const assert = require('assert');
 const {
   constants,
+  createPrivateKey,
   createSign,
   createVerify,
   generateKeyPair,
@@ -1171,3 +1172,41 @@ const sec1EncExp = (cipher) => getRegExpForPEM('EC PRIVATE KEY', cipher);
     );
   }
 }
+
+{
+  // Passing an empty passphrase string should not cause OpenSSL's default
+  // passphrase prompt in the terminal.
+  // See https://github.com/nodejs/node/issues/35898.
+
+  for (const type of ['pkcs1', 'pkcs8']) {
+    generateKeyPair('rsa', {
+      modulusLength: 1024,
+      privateKeyEncoding: {
+        type,
+        format: 'pem',
+        cipher: 'aes-256-cbc',
+        passphrase: ''
+      }
+    }, common.mustSucceed((publicKey, privateKey) => {
+      assert.strictEqual(publicKey.type, 'public');
+
+      for (const passphrase of ['', Buffer.alloc(0)]) {
+        const privateKeyObject = createPrivateKey({
+          passphrase,
+          key: privateKey
+        });
+        assert.strictEqual(privateKeyObject.asymmetricKeyType, 'rsa');
+      }
+
+      // Encrypting with an empty passphrase is not the same as not encrypting
+      // the key, and not specifying a passphrase should fail when decoding it.
+      assert.throws(() => {
+        return testSignVerify(publicKey, privateKey);
+      }, {
+        name: 'TypeError',
+        code: 'ERR_MISSING_PASSPHRASE',
+        message: 'Passphrase required for encrypted key'
+      });
+    }));
+  }
+}
