@@ -8,6 +8,7 @@ if (!common.hasCrypto)
 const assert = require('assert');
 const h2 = require('http2');
 const { kSocket } = require('internal/http2/util');
+const { kEvents } = require('internal/event_target');
 const Countdown = require('../common/countdown');
 
 {
@@ -62,7 +63,7 @@ const Countdown = require('../common/countdown');
     const req = client.request();
     req.on('error', common.expectsError({
       code: 'ERR_HTTP2_STREAM_CANCEL',
-      type: Error,
+      name: 'Error',
       message: 'The pending stream has been canceled'
     }));
 
@@ -71,36 +72,38 @@ const Countdown = require('../common/countdown');
     req.on('response', common.mustNotCall());
 
     const sessionError = {
-      type: Error,
+      name: 'Error',
       code: 'ERR_HTTP2_INVALID_SESSION',
       message: 'The session has been destroyed'
     };
 
-    common.expectsError(() => client.setNextStreamID(), sessionError);
-    common.expectsError(() => client.ping(), sessionError);
-    common.expectsError(() => client.settings({}), sessionError);
-    common.expectsError(() => client.goaway(), sessionError);
-    common.expectsError(() => client.request(), sessionError);
+    assert.throws(() => client.setNextStreamID(), sessionError);
+    assert.throws(() => client.setLocalWindowSize(), sessionError);
+    assert.throws(() => client.ping(), sessionError);
+    assert.throws(() => client.settings({}), sessionError);
+    assert.throws(() => client.goaway(), sessionError);
+    assert.throws(() => client.request(), sessionError);
     client.close();  // Should be a non-op at this point
 
     // Wait for setImmediate call from destroy() to complete
     // so that state.destroyed is set to true
     setImmediate(() => {
-      common.expectsError(() => client.setNextStreamID(), sessionError);
-      common.expectsError(() => client.ping(), sessionError);
-      common.expectsError(() => client.settings({}), sessionError);
-      common.expectsError(() => client.goaway(), sessionError);
-      common.expectsError(() => client.request(), sessionError);
+      assert.throws(() => client.setNextStreamID(), sessionError);
+      assert.throws(() => client.setLocalWindowSize(), sessionError);
+      assert.throws(() => client.ping(), sessionError);
+      assert.throws(() => client.settings({}), sessionError);
+      assert.throws(() => client.goaway(), sessionError);
+      assert.throws(() => client.request(), sessionError);
       client.close();  // Should be a non-op at this point
     });
 
     req.resume();
-    req.on('end', common.mustCall());
+    req.on('end', common.mustNotCall());
     req.on('close', common.mustCall(() => server.close()));
   }));
 }
 
-// test destroy before goaway
+// Test destroy before goaway
 {
   const server = h2.createServer();
   server.on('stream', common.mustCall((stream) => {
@@ -120,7 +123,7 @@ const Countdown = require('../common/countdown');
   }));
 }
 
-// test destroy before connect
+// Test destroy before connect
 {
   const server = h2.createServer();
   server.on('stream', common.mustNotCall());
@@ -138,23 +141,24 @@ const Countdown = require('../common/countdown');
   }));
 }
 
-// test close before connect
+// Test close before connect
 {
   const server = h2.createServer();
 
   server.on('stream', common.mustNotCall());
   server.listen(0, common.mustCall(() => {
     const client = h2.connect(`http://localhost:${server.address().port}`);
+    client.on('close', common.mustCall());
     const socket = client[kSocket];
     socket.on('close', common.mustCall(() => {
       assert(socket.destroyed);
     }));
 
     const req = client.request();
-    // should throw goaway error
+    // Should throw goaway error
     req.on('error', common.expectsError({
       code: 'ERR_HTTP2_GOAWAY_SESSION',
-      type: Error,
+      name: 'Error',
       message: 'New streams cannot be created after receiving a GOAWAY'
     }));
 
@@ -164,3 +168,76 @@ const Countdown = require('../common/countdown');
     req.on('close', common.mustCall(() => server.close()));
   }));
 }
+
+// Destroy with AbortSignal
+{
+  const server = h2.createServer();
+  const controller = new AbortController();
+
+  server.on('stream', common.mustNotCall());
+  server.listen(0, common.mustCall(() => {
+    const client = h2.connect(`http://localhost:${server.address().port}`);
+    client.on('close', common.mustCall());
+
+    const { signal } = controller;
+    assert.strictEqual(signal[kEvents].get('abort'), undefined);
+
+    client.on('error', common.mustCall(() => {
+      // After underlying stream dies, signal listener detached
+      assert.strictEqual(signal[kEvents].get('abort'), undefined);
+    }));
+
+    const req = client.request({}, { signal });
+
+    req.on('error', common.mustCall((err) => {
+      assert.strictEqual(err.code, 'ABORT_ERR');
+      assert.strictEqual(err.name, 'AbortError');
+    }));
+    req.on('close', common.mustCall(() => server.close()));
+
+    assert.strictEqual(req.aborted, false);
+    assert.strictEqual(req.destroyed, false);
+    // Signal listener attached
+    assert.strictEqual(signal[kEvents].get('abort').size, 1);
+
+    controller.abort();
+
+    assert.strictEqual(req.aborted, false);
+    assert.strictEqual(req.destroyed, true);
+  }));
+}
+// Pass an already destroyed signal to abort immediately.
+{
+  const server = h2.createServer();
+  const controller = new AbortController();
+
+  server.on('stream', common.mustNotCall());
+  server.listen(0, common.mustCall(() => {
+    const client = h2.connect(`http://localhost:${server.address().port}`);
+    client.on('close', common.mustCall());
+
+    const { signal } = controller;
+    controller.abort();
+
+    assert.strictEqual(signal[kEvents].get('abort'), undefined);
+
+    client.on('error', common.mustCall(() => {
+      // After underlying stream dies, signal listener detached
+      assert.strictEqual(signal[kEvents].get('abort'), undefined);
+    }));
+
+    const req = client.request({}, { signal });
+    // Signal already aborted, so no event listener attached.
+    assert.strictEqual(signal[kEvents].get('abort'), undefined);
+
+    assert.strictEqual(req.aborted, false);
+    // Destroyed on same tick as request made
+    assert.strictEqual(req.destroyed, true);
+
+    req.on('error', common.mustCall((err) => {
+      assert.strictEqual(err.code, 'ABORT_ERR');
+      assert.strictEqual(err.name, 'AbortError');
+    }));
+    req.on('close', common.mustCall(() => server.close()));
+  }));
+}
