@@ -24,6 +24,7 @@
 const common = require('./common.js');
 const fs = require('fs');
 const unified = require('unified');
+const find = require('unist-util-find');
 const visit = require('unist-util-visit');
 const markdown = require('remark-parse');
 const gfm = require('remark-gfm');
@@ -66,27 +67,6 @@ const gtocHTML = unified()
 const templatePath = path.join(docPath, 'template.html');
 const template = fs.readFileSync(templatePath, 'utf8');
 
-function processContent(content) {
-  content = content.toString();
-  // Increment header tag levels to avoid multiple h1 tags in a doc.
-  // This means we can't already have an <h6>.
-  if (content.includes('<h6>')) {
-    throw new Error('Cannot increment a level 6 header');
-  }
-  // `++level` to convert the string to a number and increment it.
-  content = content.replace(/(?<=<\/?h)[1-5](?=[^<>]*>)/g, (level) => ++level);
-  // Wrap h3 tags in section tags.
-  let firstTime = true;
-  return content
-    .replace(/<h3/g, (heading) => {
-      if (firstTime) {
-        firstTime = false;
-        return '<section>' + heading;
-      }
-      return '</section><section>' + heading;
-    }) + (firstTime ? '' : '</section>');
-}
-
 function toHTML({ input, content, filename, nodeVersion, versions }) {
   filename = path.basename(filename, '.md');
 
@@ -100,7 +80,7 @@ function toHTML({ input, content, filename, nodeVersion, versions }) {
                      .replace('__GTOC__', gtocHTML.replace(
                        `class="nav-${id}"`, `class="nav-${id} active"`))
                      .replace('__EDIT_ON_GITHUB__', editOnGitHub(filename))
-                     .replace('__CONTENT__', processContent(content));
+                     .replace('__CONTENT__', content.toString());
 
   const docCreated = input.match(
     /<!--\s*introduced_in\s*=\s*v([0-9]+)\.([0-9]+)\.[0-9]+\s*-->/);
@@ -117,13 +97,7 @@ function toHTML({ input, content, filename, nodeVersion, versions }) {
 // Set the section name based on the first header.  Default to 'Index'.
 function firstHeader() {
   return (tree, file) => {
-    let heading;
-    visit(tree, (node) => {
-      if (node.type === 'heading') {
-        heading = node;
-        return false;
-      }
-    });
+    const heading = find(tree, { type: 'heading' });
 
     if (heading && heading.children.length) {
       const recursiveTextContent = (node) =>
@@ -234,9 +208,9 @@ function preprocessElements({ filename }) {
           const [, prefix, number, explication] =
             text.value.match(STABILITY_RE);
 
-          // Stability indices are never more than 3 nodes away from their
-          // heading.
-          const isStabilityIndex = index - headingIndex <= 3;
+          const isStabilityIndex =
+            index - 2 === headingIndex || // General.
+            index - 3 === headingIndex;   // With api_metadata block.
 
           if (heading && isStabilityIndex) {
             heading.stability = number;
@@ -246,7 +220,7 @@ function preprocessElements({ filename }) {
 
           // Do not link to the section we are already in.
           const noLinking = filename.includes('documentation') &&
-            heading !== null && heading.children[0].value === 'Stability index';
+            heading !== null && heading.children[0].value === 'Stability Index';
 
           // Collapse blockquote and paragraph into a single node
           node.type = 'paragraph';
@@ -408,19 +382,13 @@ function buildToc({ filename, apilinks }) {
       node.children.push({ type: 'html', value: anchor });
     });
 
-    if (toc !== '') {
-      file.toc = '<details id="toc" open><summary>Table of contents</summary>' +
-        unified()
-          .use(markdown)
-          .use(gfm)
-          .use(remark2rehype, { allowDangerousHtml: true })
-          .use(raw)
-          .use(htmlStringify)
-          .processSync(toc).toString() +
-        '</details>';
-    } else {
-      file.toc = '<!-- TOC -->';
-    }
+    file.toc = unified()
+      .use(markdown)
+      .use(gfm)
+      .use(remark2rehype, { allowDangerousHtml: true })
+      .use(raw)
+      .use(htmlStringify)
+      .processSync(toc).toString();
   };
 }
 
